#
#
# $Id: util2ch.rb,v 1.72 2003/12/13 13:25:15 Yuukis Exp $
#
#

module String2ch
	require 'nkf'

	ConvertStrings = { '&gt;' => '>', '&lt;' => '<' }
	RegexDat = /(.*)<>(.*)<>(.*)<>\s(.*)\s<>/

	def String2ch.dat2str(str, lastnum)
		x = ''
		lastnum += 1
		str.each {|line|
			NKF.nkf('-exS', line).gsub!(RegexDat) { x << fmt(lastnum, $1, $2, $3, $4) }
			lastnum += 1
		}
		return(x)
	end

	def String2ch.dat2txt!(str, sbj = false)
		# ConvertStrings = { '&gt;' => '>', '&lt;' => '<' }
		ConvertStrings.each {|from, to| str.gsub!(from, to)}
		unless sbj
			str.gsub!(/&nbsp;/, ' ')
			str.gsub!(/&thinsp;/, ' ')
			str.gsub!(/&amp;/, '&')
			str.gsub!(/&quot;/, '"')
			str.gsub!(/ <br> /, "\n")
			str.gsub!(/<br>/, "\n")
			str.gsub!(/<a href=.*?>/, '')
			str.gsub!(/<\/a>/, '')
			str.gsub!(/^ +/, '')
			str.gsub!(/ $/, '')
			str.gsub!(/ +/, ' ')
		end
		return(str)
	end

	def String2ch.get_title(fn)
		line = ''
		open(fn) {|x| while line = x.gets do ; break ; end }
		dat2txt!(NKF.nkf('-exS', line.split('<>').pop).chomp, true)
	end

	def String2ch.fmt(num, name, mailto, date, message)
		line = ''
		catch(:hit_ng) {
			CFG::Net.ng_word.each_pair {|word, key|
				case key
				when 'name'
					if name =~ word
						line << "#{num}: #{CFG::NLS.abone} <#{$&}> :#{date}\n-----\n"
						throw(:hit_ng, line)
					end
				when 'mailto'
					if mailto =~ word
						line << "#{num}: #{CFG::NLS.abone} <#{$&}> :#{date}\n-----\n"
						throw(:hit_ng, line)
					end
				when 'message'
					if message =~ word
						line << "#{num}: #{CFG::NLS.abone} <#{$&}> :#{date}\n-----\n"
						throw(:hit_ng, line)
					end
				when 'all'
					if name =~ word or mailto =~ word or message =~ word
						line << "#{num}: #{CFG::NLS.abone} <#{$&}> :#{date}\n-----\n"
						throw(:hit_ng, line)
					end
				end
			}
			line << "#{num}: #{dat2txt!(name)} <#{mailto}> : #{date}\n\n"
			line << "#{dat2txt!(message).gsub!(/^/, ' ')}\n\n-----\n"
		}
		return(line)
	end

	def String2ch.parse_uri(uri)  # uri == http://pc.2ch.net/unix/
		server = ''
		board = ''
		uri.sub(/http:\/\/(.*)\/(.*)\/.*/) {
			server = $1
			board = $2
		}
		return([server, board])
	end

	def String2ch.trunc(str)  # From goRua. Thanks.
		if  /^(.{1,#{CFG::UIF.tab_width}}).*$/ =~ str
			return $1
		else
			return str
		end
	end
end


class SpHandles  # Bookmark & Abone
	def initialize(board, dir_name = false)
		if dir_name
			@fn = "#{CFG::SYS.root}#{board}/#{dir_name}/#{self.class.to_s.downcase}.dat"
		else
			@fn = "#{CFG::SYS.root}#{board}/#{self.class.to_s.downcase}.dat"
		end
		update
	end

	def update
		@values = []
		if File.exist?(@fn)
			a = IO.readlines(@fn)
			a.each {|th| @values << th.chomp }
		end
	end

	def each(&block)
		@values.each(&block)
	end

	def include?(dat)
		return(@values.include?(dat))
	end

	def add(dat)
		@values << dat.chomp
		write
	end

	def delete(dat)
		@values.delete(dat)
		write
	end

	def toggle(dat)
		if include?(dat)
			delete(dat)
		else
			add(dat)
		end
	end

	def write
		str = ''
		@values.each {|line| str << line + "\n" }
		fh = open(@fn, 'w')
		fh.write str
		fh.close
		update
	end
	private :write
end


class Bookmark < SpHandles
	def initialize(board, dir_name = false)
		super
	end
end


class Abone < SpHandles
	def initialize(board, dir_name = false)
		if File.exist?("#{CFG::SYS.root}#{board}/abone.dat")  # NR.1 Ѵ
			a = IO.readlines("#{CFG::SYS.root}#{board}/abone.dat").join
			if a =~ /<>/
				str = ''
				a.each {|th| str << th.chomp.split('<>')[0] + "\n" }
				fh = open("#{CFG::SYS.root}#{board}/abone.dat", 'w')
				fh.write str
				fh.close
			end
		end
		super
	end
end


module Trash
	TRASH = 'trash'

	def Trash.drop(fn)
		dir = CFG::SYS.root + '.' + TRASH + '/'
		Dir.mkdir(dir) unless File.exist?(dir)
		File.rename(fn, dir + File.basename(fn))
	end
end


module Searched
	DAT = 'searched.dat'

	def get_searched(board)
		return(['']) unless File.exist?(CFG::SYS.root + board + '/' + DAT)
		a = []
		IO.readlines(CFG::SYS.root + board + '/' + DAT).each {|x| a << x.chomp }
		return(a) if a.size == 1
		return(a.reverse!)
	end

	def add_searched(board, str)
		a = []
		if File.exist?(CFG::SYS.root + board + '/' + DAT)
			IO.readlines(CFG::SYS.root + board + '/' + DAT).each {|x| a << x.chomp }
		end
		if a.include?(str)
			a.delete(str)
		else
			while a.size >= CFG::UIF.searched_max
				a.shift
			end
		end
		a << str
		fh = open(CFG::SYS.root + board + '/' + DAT, 'w')
		fh.write a.join("\n")
		fh.close
	end
end


module Logger
	def Logger.init(app_name)
		puts app_name
		puts '--------------------'
	end

	def Logger.write(fn, ln, str)
		puts "#{File.basename(fn)}: #{ln}: #{str}"
	end
end


class Status
	def initialize
		@status = false
	end

	def toggleStatus
		if @status == true
			@status = false
		else
			@status = true
		end
		return(@status)
	end

	def active?
		@status
	end
end
